import urllib.request as req
import urllib.parse
from bs4 import BeautifulSoup
import re
import collections

Person = collections.namedtuple('Person', "Name Bio Path")
Film = collections.namedtuple("Film", "Title Year")

class ImdbSearch:
    m_ImdbSearchMovie = "https://www.imdb.com/filmosearch/?explore=title_type&role=%s&sort=year,desc&mode=simple&page=%d&title_type=movie"
    m_ImdbSearchPersonUrl = "https://www.imdb.com/search/name/?name=%s&count=100"

    def __init__(self):
        #add proxy stuff here if u need to
        opener = req.build_opener(req.HTTPHandler)
        req.install_opener(opener)

    def FindPerson(self, Name):
        #send request to build list of people
        url = self.m_ImdbSearchPersonUrl % urllib.parse.quote(Name)
        resp = self.SendRequest(url)

        #find the tags for all returned names in the response
        soup = BeautifulSoup(resp, 'html.parser')
        listerItems = soup.find_all("div", class_="lister-item-content")

        #build result list
        result = list()
        for listerItem in listerItems:
            pElem = listerItem.find_all("p", class_="")
            if (len(pElem) > 0): # some people dont have a bio so we gotta check that
                Bio = pElem[0].text.strip()
            else:
                Bio = None
            
            #clean up and save results
            PersonName = listerItem.a.string.strip()
            Path = listerItem.a['href'][6:].strip() #strip the /name/
            result.append(Person(PersonName, Bio, Path))

        return result

    def GetMovies(self, PersonPath):
        page = 0
        result = list()

        #only 50 shown per page so need to loop
        while True:
            page += 1
            url = self.m_ImdbSearchMovie % (PersonPath, page)
            resp = self.SendRequest(url)

            #no result means we gone over the last page
            if(resp == "" or "No results. Try removing genres, ratings, or other filters to see more." in resp):
                break

            # get list of movies from the response
            soup = BeautifulSoup(resp, 'html.parser')
            films = soup.find_all("span", class_="lister-item-header")
            
            # build dict of movies with the year dict will be in decending order
            for film in films:
                year = film.find("span", class_="lister-item-year text-muted unbold").string
                if(year != None): # if the year is none then the movie has not been released yet
                    years = re.findall("\d{4}", year)
                    if(len(years) > 0):
                        year = years[0]
                    else: # year is in an unkown format or something wierd 
                        year = "Unknown"
                else:
                    year = "TBA"

                result.append(Film(film.a.string.strip(), year))

        return result

    def SendRequest(self, URL):
        result = ""
        try:
            conn = req.urlopen(URL)
            result = conn.read().decode("utf8")

        except:
            print("error sending request to: %s" % URL)

        return result



from .IMDB import *
import json
import urllib.request

class Console:
    prompt = "IMDB>"
    imdb = None

    def __init__(self):
        self.imdb = ImdbSearch()
        return

    def run(self):
        Name = self.GetSearchName()
        people = self.imdb.FindPerson(Name)

        person = None
        if(len(people) > 0):
            if(len(people) == 1):
                #only one person returned so select them
                person = people[0]
            else:
                #multiple people returned so prompt for which one
                person = self.SelectPersonFromList(people)
        else:
            print("no results for %s" % Name)
            return
        
        #print out the movies the person is in
        print("%s\n%s" % (person.Name, person.Bio))
        movies = self.imdb.GetMovies(person.Path)
        self.PrintFilms(movies)

        #ask if wanna save to json file
        print("Save to File? Y / N (default)")
        choice = input(self.prompt)
        if(choice.lower() == 'y'):
            self.SaveToJson(person, movies)

    def GetSearchName(self):
        print("Input a name to search")
        return input(self.prompt)
    
    def SelectPersonFromList(self, people):
        #list all people with index value
        id = 0
        for p in people:
            print("%d.\nName: %s\nBio: %s\n\n" % (id, p.Name, p.Bio))
            id += 1

        #select person by index value
        result = None
        while True:
            print("enter the number of the person to look up")
          
            try:
                ChosenInt = int(input(self.prompt))
                if (ChosenInt < len(people) and ChosenInt > -1):
                    result = people[ChosenInt]
                    break

            except ValueError:
                continue

        return result

    def PrintFilms(self, Films):
        print("What date order would you like films in?\nA: Ascending (default)\nD: Descending")
        choice = input(self.prompt)
        # films are in descending order already so if we get anything but a d reverse the order
        if(choice.lower() != "d"):
            Films.reverse()

        #print out the films
        for film in Films:
           print("%s\t\t%s" % (film.Year, film.Title))

    def SaveToJson(self, Actor, Films):
        #get file path
        print("Enter the file to save to")
        filepath= input(self.prompt)

        #make it pretty
        data = {}
        data["Person"] = Actor._asdict()
        data["Films"] = list()
        for film in Films:
            data["Films"].append(film._asdict())

        try:
            with open(filepath, 'w') as outfile:
                json.dump(data, outfile, indent=4)
        except:
            print("Error writing to file %s. Do you have permissions?" % choice)



from setuptools import setup
setup(
    name = 'imdbwho',
    version = '0.1.0',
    packages = ['imdbwho'],
    license='MIT',
    install_requires=[
        'beautifulsoup4'
    ],

    entry_points = {
        'console_scripts': [
            'imdbwho = imdbwho.__main__:main'
        ]
    })
